@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: 3cd97702b33ee1b8354a06a92700bd8fcff0e37e $ */
+/* $Id: cd53f4adf0f7fcb974154ce418a71b2ce1589268 $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -42,7 +42,7 @@
 /* {{{ inifile_version */
 char *inifile_version() 
 {
-	return "1.0, $Id: 3cd97702b33ee1b8354a06a92700bd8fcff0e37e $";
+	return "1.0, $Id: cd53f4adf0f7fcb974154ce418a71b2ce1589268 $";
 }
 /* }}} */ 
 
@@ -460,7 +460,7 @@ static int inifile_filter(inifile *dba, inifile *from, const key_type *key TSRML
  */
 static int inifile_delete_replace_append(inifile *dba, const key_type *key, const val_type *value, int append TSRMLS_DC) 
 {
-	size_t pos_grp_start = 0, pos_grp_next;
+	size_t pos_grp_start, pos_grp_next;
 	inifile *ini_tmp = NULL;
 	php_stream *fp_tmp = NULL;
 	int ret;
