@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2013 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -343,6 +343,7 @@ static int pdo_mysql_stmt_next_rowset(pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 	pdo_mysql_stmt *S = (pdo_mysql_stmt*)stmt->driver_data;
 	pdo_mysql_db_handle *H = S->H;
 	long row_count;
+	int ret;
 	PDO_DBG_ENTER("pdo_mysql_stmt_next_rowset");
 	PDO_DBG_INF_FMT("stmt=%p", S->stmt);
 
@@ -411,21 +412,26 @@ static int pdo_mysql_stmt_next_rowset(pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 		S->result = NULL;
 	}
 
-	if (!mysql_more_results(H->server)) {
-		/* No more results */
-		PDO_DBG_RETURN(0);	
-	}
+	ret = mysql_next_result(H->server);
 #if PDO_USE_MYSQLND
-	if (mysql_next_result(H->server) == FAIL) {
+	/* for whatever reason mysqlnd breaks with libmysql compatibility at the C level, no -1 */
+	if (PASS != ret) {
 		pdo_mysql_error_stmt(stmt);
 		PDO_DBG_RETURN(0);
-	} else {
+	}
+	if (mysql_more_results(H->server)) {
 		PDO_DBG_RETURN(pdo_mysql_fill_stmt_from_result(stmt TSRMLS_CC));
+	} else {
+		/* No more results */
+		PDO_DBG_RETURN(0);
 	}
 #else
-	if (mysql_next_result(H->server) > 0) {
+	if (ret > 0) {
 		pdo_mysql_error_stmt(stmt);
 		PDO_DBG_RETURN(0);
+	} else if (ret < 0) {
+		/* No more results */
+		PDO_DBG_RETURN(0);
 	} else {
 		PDO_DBG_RETURN(pdo_mysql_fill_stmt_from_result(stmt TSRMLS_CC));
 	}
@@ -761,11 +767,8 @@ static char *type_to_name_native(int type) /* {{{ */
     switch (type) {
         PDO_MYSQL_NATIVE_TYPE_NAME(STRING)
         PDO_MYSQL_NATIVE_TYPE_NAME(VAR_STRING)
-#ifdef FIELD_TYPE_TINY
+#ifdef MYSQL_HAS_TINY
         PDO_MYSQL_NATIVE_TYPE_NAME(TINY)
-#endif
-#ifdef FIELD_TYPE_BIT
-        PDO_MYSQL_NATIVE_TYPE_NAME(BIT)
 #endif
         PDO_MYSQL_NATIVE_TYPE_NAME(SHORT)
         PDO_MYSQL_NATIVE_TYPE_NAME(LONG)
@@ -781,7 +784,7 @@ static char *type_to_name_native(int type) /* {{{ */
         PDO_MYSQL_NATIVE_TYPE_NAME(GEOMETRY)
 #endif
         PDO_MYSQL_NATIVE_TYPE_NAME(TIMESTAMP)
-#ifdef FIELD_TYPE_YEAR
+#ifdef MYSQL_HAS_YEAR
         PDO_MYSQL_NATIVE_TYPE_NAME(YEAR)
 #endif
         PDO_MYSQL_NATIVE_TYPE_NAME(SET)
