@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -115,10 +115,6 @@
 #endif
 /* }}} */
 
-#ifndef S_ISREG
-#define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
-#endif
-
 PHPAPI int (*php_register_internal_extensions_func)(TSRMLS_D) = php_register_internal_extensions;
 
 #ifndef ZTS
@@ -261,7 +257,7 @@ static void php_disable_classes(TSRMLS_D)
 
 /* {{{ php_binary_init
  */
-static void php_binary_init(TSRMLS_D)
+static void php_binary_init(TSRMLS_D) 
 {
 	char *binary_location;
 #ifdef PHP_WIN32
@@ -280,14 +276,13 @@ static void php_binary_init(TSRMLS_D)
 			if ((envpath = getenv("PATH")) != NULL) {
 				char *search_dir, search_path[MAXPATHLEN];
 				char *last = NULL;
-				struct stat s;
 
 				path = estrdup(envpath);
 				search_dir = php_strtok_r(path, ":", &last);
 
 				while (search_dir) {
 					snprintf(search_path, MAXPATHLEN, "%s/%s", search_dir, sapi_module.executable_location);
-					if (VCWD_REALPATH(search_path, binary_location) && !VCWD_ACCESS(binary_location, X_OK) && VCWD_STAT(binary_location, &s) == 0 && S_ISREG(s.st_mode)) {
+					if (VCWD_REALPATH(search_path, binary_location) && !VCWD_ACCESS(binary_location, X_OK)) {
 						found = 1;
 						break;
 					}
@@ -789,9 +784,6 @@ PHPAPI void php_verror(const char *docref, const char *params, int type, const c
 	/* no docref given but function is known (the default) */
 	if (!docref && is_function) {
 		int doclen;
-		while (*function == '_') {
-			function++;
-		}
 		if (space[0] == '\0') {
 			doclen = spprintf(&docref_buf, 0, "function.%s", function);
 		} else {
@@ -853,7 +845,7 @@ PHPAPI void php_verror(const char *docref, const char *params, int type, const c
 		efree(docref_buf);
 	}
 
-	if (PG(track_errors) && module_initialized &&
+	if (PG(track_errors) && module_initialized && 
 			(!EG(user_error_handler) || !(EG(user_error_handler_error_reporting) & type))) {
 		if (!EG(active_symbol_table)) {
 			zend_rebuild_symbol_table(TSRMLS_C);
@@ -970,7 +962,7 @@ static void php_error_cb(int type, const char *error_filename, const uint error_
 	/* store the error if it has changed */
 	if (display) {
 #ifdef ZEND_SIGNALS
-		HANDLE_BLOCK_INTERRUPTIONS();
+		HANDLE_BLOCK_INTERRUPTIONS();	
 #endif
 		if (PG(last_error_message)) {
 			free(PG(last_error_message));
@@ -1141,20 +1133,11 @@ static void php_error_cb(int type, const char *error_filename, const uint error_
 		case E_PARSE:
 		case E_COMPILE_ERROR:
 		case E_USER_ERROR:
-		{ /* new block to allow variable definition */
-			/* eval() errors do not affect exit_status or response code */
-			zend_bool during_eval = (type == E_PARSE) && (EG(current_execute_data) &&
-						EG(current_execute_data)->opline &&
-						EG(current_execute_data)->opline->opcode == ZEND_INCLUDE_OR_EVAL &&
-						EG(current_execute_data)->opline->extended_value == ZEND_EVAL);
-			if (!during_eval) {
-				EG(exit_status) = 255;
-			}
+			EG(exit_status) = 255;
 			if (module_initialized) {
 				if (!PG(display_errors) &&
 				    !SG(headers_sent) &&
-					SG(sapi_headers).http_response_code == 200 &&
-				    !during_eval
+					SG(sapi_headers).http_response_code == 200
 				) {
 					sapi_header_line ctr = {0};
 
@@ -1175,7 +1158,6 @@ static void php_error_cb(int type, const char *error_filename, const uint error_
 				}
 			}
 			break;
-		}
 	}
 
 	/* Log if necessary */
@@ -1229,7 +1211,7 @@ PHPAPI char *php_get_current_user(TSRMLS_D)
 		name[len] = '\0';
 		SG(request_info).current_user_length = len;
 		SG(request_info).current_user = estrndup(name, len);
-		return SG(request_info).current_user;
+		return SG(request_info).current_user;		
 #else
 		struct passwd *pwd;
 #if defined(ZTS) && defined(HAVE_GETPWUID_R) && defined(_SC_GETPW_R_SIZE_MAX)
@@ -1257,9 +1239,9 @@ PHPAPI char *php_get_current_user(TSRMLS_D)
 #if defined(ZTS) && defined(HAVE_GETPWUID_R) && defined(_SC_GETPW_R_SIZE_MAX)
 		efree(pwbuf);
 #endif
-		return SG(request_info).current_user;
+		return SG(request_info).current_user;		
 #endif
-	}
+	}	
 }
 /* }}} */
 
@@ -1274,7 +1256,7 @@ PHP_FUNCTION(set_time_limit)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &new_timeout) == FAILURE) {
 		return;
 	}
-
+	
 	new_timeout_strlen = zend_spprintf(&new_timeout_str, 0, "%ld", new_timeout);
 
 	if (zend_alter_ini_entry_ex("max_execution_time", sizeof("max_execution_time"), new_timeout_str, new_timeout_strlen, PHP_INI_USER, PHP_INI_STAGE_RUNTIME, 0 TSRMLS_CC) == SUCCESS) {
@@ -1531,7 +1513,7 @@ int php_request_startup(TSRMLS_D)
 	int retval = SUCCESS;
 
 #ifdef HAVE_DTRACE
-	DTRACE_REQUEST_STARTUP(SAFE_FILENAME(SG(request_info).path_translated), SAFE_FILENAME(SG(request_info).request_uri), (char *)SAFE_FILENAME(SG(request_info).request_method));
+	DTRACE_REQUEST_STARTUP(SAFE_FILENAME(SG(request_info).path_translated), SAFE_FILENAME(SG(request_info).request_uri), SAFE_FILENAME(SG(request_info).request_method));
 #endif /* HAVE_DTRACE */
 
 #ifdef PHP_WIN32
@@ -1841,7 +1823,7 @@ void php_request_shutdown(void *dummy)
 #endif
 
 #ifdef HAVE_DTRACE
-	DTRACE_REQUEST_SHUTDOWN(SAFE_FILENAME(SG(request_info).path_translated), SAFE_FILENAME(SG(request_info).request_uri), (char *)SAFE_FILENAME(SG(request_info).request_method));
+	DTRACE_REQUEST_SHUTDOWN(SAFE_FILENAME(SG(request_info).path_translated), SAFE_FILENAME(SG(request_info).request_uri), SAFE_FILENAME(SG(request_info).request_method));
 #endif /* HAVE_DTRACE */
 }
 /* }}} */
@@ -1908,7 +1890,7 @@ static void core_globals_dtor(php_core_globals *core_globals TSRMLS_DC)
 PHP_MINFO_FUNCTION(php_core) { /* {{{ */
 	php_info_print_table_start();
 	php_info_print_table_row(2, "PHP Version", PHP_VERSION);
-	php_info_print_table_end();
+	php_info_print_table_end(); 
 	DISPLAY_INI_ENTRIES();
 }
 /* }}} */
@@ -2078,8 +2060,6 @@ int php_module_startup(sapi_module_struct *sf, zend_module_entry *additional_mod
 	EG(exception_class) = NULL;
 	PG(disable_functions) = NULL;
 	PG(disable_classes) = NULL;
-	EG(exception) = NULL;
-	EG(objects_store).object_buckets = NULL;
 
 #if HAVE_SETLOCALE
 	setlocale(LC_CTYPE, "");
@@ -2186,7 +2166,7 @@ int php_module_startup(sapi_module_struct *sf, zend_module_entry *additional_mod
 		return FAILURE;
 	}
 
-	/* initialize registry for images to be used in phpinfo()
+	/* initialize registry for images to be used in phpinfo() 
 	   (this uses configuration parameters from php.ini)
 	 */
 	if (php_init_info_logos() == FAILURE) {
@@ -2232,7 +2212,7 @@ int php_module_startup(sapi_module_struct *sf, zend_module_entry *additional_mod
 			EG(current_module) = NULL;
 		}
 	}
-
+	
 	/* disable certain classes and functions as requested by php.ini */
 	php_disable_functions(TSRMLS_C);
 	php_disable_classes(TSRMLS_C);
@@ -2267,38 +2247,38 @@ int php_module_startup(sapi_module_struct *sf, zend_module_entry *additional_mod
 			const char *directives[16]; /* Remember to change this if the number of directives change */
 		} directives[2] = {
 			{
-				E_DEPRECATED,
-				"Directive '%s' is deprecated in PHP 5.3 and greater",
+				E_DEPRECATED, 
+				"Directive '%s' is deprecated in PHP 5.3 and greater", 
 				{
 					NULL
 				}
-			},
+			}, 
 			{
-				E_CORE_ERROR,
-				"Directive '%s' is no longer available in PHP",
+				E_CORE_ERROR, 
+				"Directive '%s' is no longer available in PHP", 
 				{
 					"allow_call_time_pass_reference",
-					"define_syslog_variables",
-					"highlight.bg",
-					"magic_quotes_gpc",
-					"magic_quotes_runtime",
-					"magic_quotes_sybase",
-					"register_globals",
-					"register_long_arrays",
-					"safe_mode",
-					"safe_mode_gid",
-					"safe_mode_include_dir",
-					"safe_mode_exec_dir",
-					"safe_mode_allowed_env_vars",
-					"safe_mode_protected_env_vars",
-					"zend.ze1_compatibility_mode",
+					"define_syslog_variables", 
+					"highlight.bg", 
+					"magic_quotes_gpc", 
+					"magic_quotes_runtime", 
+					"magic_quotes_sybase", 
+					"register_globals", 
+					"register_long_arrays", 
+					"safe_mode", 
+					"safe_mode_gid", 
+					"safe_mode_include_dir", 
+					"safe_mode_exec_dir", 
+					"safe_mode_allowed_env_vars", 
+					"safe_mode_protected_env_vars", 
+					"zend.ze1_compatibility_mode", 
 					NULL
 				}
 			}
 		};
 
 		unsigned int i;
-
+		
 		zend_try {
 			/* 2 = Count of deprecation structs */
 			for (i = 0; i < 2; i++) {
@@ -2318,7 +2298,7 @@ int php_module_startup(sapi_module_struct *sf, zend_module_entry *additional_mod
 			retval = FAILURE;
 		} zend_end_try();
 	}
-
+	
 	sapi_deactivate(TSRMLS_C);
 	module_startup = 0;
 
@@ -2373,7 +2353,7 @@ void php_module_shutdown(TSRMLS_D)
 	sapi_flush(TSRMLS_C);
 
 	zend_shutdown(TSRMLS_C);
-
+	
 	/* Destroys filter & transport registries too */
 	php_shutdown_stream_wrappers(module_number TSRMLS_CC);
 
@@ -2417,7 +2397,7 @@ PHPAPI int php_execute_script(zend_file_handle *primary_file TSRMLS_DC)
 	zend_file_handle *prepend_file_p, *append_file_p;
 	zend_file_handle prepend_file = {0}, append_file = {0};
 #if HAVE_BROKEN_GETCWD 
-	volatile int old_cwd_fd = -1;
+	int old_cwd_fd = -1;
 #else
 	char *old_cwd;
 	ALLOCA_FLAG(use_heap)
