@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,17 +20,15 @@
 /* $Id: php_cli.c 306938 2011-01-01 02:17:06Z felipe $ */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <fcntl.h>
 #include <assert.h>
 
 #ifdef PHP_WIN32
-# include <process.h>
-# include <io.h>
-# include "win32/time.h"
-# include "win32/signal.h"
-# include "win32/php_registry.h"
-# include <sys/timeb.h>
+#include <process.h>
+#include <io.h>
+#include "win32/time.h"
+#include "win32/signal.h"
+#include "win32/php_registry.h"
 #else
 # include "php_config.h"
 #endif
@@ -193,17 +191,17 @@ typedef struct php_cli_server {
 	HashTable clients;
 } php_cli_server;
 
-typedef struct php_cli_server_http_response_status_code_pair {
+typedef struct php_cli_server_http_reponse_status_code_pair {
 	int code;
 	const char *str;
-} php_cli_server_http_response_status_code_pair;
+} php_cli_server_http_reponse_status_code_pair;
 
 typedef struct php_cli_server_ext_mime_type_pair {
 	const char *ext;
 	const char *mime_type;
 } php_cli_server_ext_mime_type_pair;
 
-static php_cli_server_http_response_status_code_pair status_map[] = {
+static php_cli_server_http_reponse_status_code_pair status_map[] = {
 	{ 100, "Continue" },
 	{ 101, "Switching Protocols" },
 	{ 200, "OK" },
@@ -238,19 +236,15 @@ static php_cli_server_http_response_status_code_pair status_map[] = {
 	{ 415, "Unsupported Media Type" },
 	{ 416, "Requested Range Not Satisfiable" },
 	{ 417, "Expectation Failed" },
-	{ 428, "Precondition Required" },
-	{ 429, "Too Many Requests" },
-	{ 431, "Request Header Fields Too Large" },
 	{ 500, "Internal Server Error" },
 	{ 501, "Not Implemented" },
 	{ 502, "Bad Gateway" },
 	{ 503, "Service Unavailable" },
 	{ 504, "Gateway Timeout" },
 	{ 505, "HTTP Version Not Supported" },
-	{ 511, "Network Authentication Required" },
 };
 
-static php_cli_server_http_response_status_code_pair template_map[] = {
+static php_cli_server_http_reponse_status_code_pair template_map[] = {
 	{ 400, "<h1>%s</h1><p>Your browser sent a request that this server could not understand.</p>" },
 	{ 404, "<h1>%s</h1><p>The requested resource %s was not found on this server.</p>" },
 	{ 500, "<h1>%s</h1><p>The server is temporarily unavailable.</p>" },
@@ -265,13 +259,10 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "gif", "image/gif" },
 	{ "jpg", "image/jpeg" },
 	{ "jpeg", "image/jpeg" },
-	{ "jpe", "image/jpeg" },
 	{ "png", "image/png" },
+	{ "jpe", "image/jpeg" },
 	{ "svg", "image/svg+xml" },
 	{ "txt", "text/plain" },
-	{ "webm", "video/webm" },
-	{ "ogv", "video/ogg" },
-	{ "ogg", "audio/ogg" },
 	{ NULL, NULL }
 };
 
@@ -294,36 +285,6 @@ static const char php_cli_server_css[] = "<style>\n" \
 										"</style>\n";
 /* }}} */
 
-#ifdef PHP_WIN32
-int php_cli_server_get_system_time(char *buf) {
-	struct _timeb system_time;
-	errno_t err;
-
-	if (buf == NULL) {
-		return -1;
-	}
-
-	_ftime(&system_time);
-	err = ctime_s(buf, 52, &(system_time.time) );
-	if (err) {
-		return -1;
-	}
-	return 0;
-}
-#else
-int php_cli_server_get_system_time(char *buf) {
-	struct timeval tv;
-	struct tm tm;
-
-	gettimeofday(&tv, NULL);
-
-	/* TODO: should be checked for NULL tm/return vaue */
-	php_localtime_r(&tv.tv_sec, &tm);
-	php_asctime_r(&tm, buf);
-	return 0;
-}
-#endif
-
 static void char_ptr_dtor_p(char **p) /* {{{ */
 {
 	pefree(*p, 1);
@@ -334,43 +295,28 @@ static char *get_last_error() /* {{{ */
 	return pestrdup(strerror(errno), 1);
 } /* }}} */
 
-static int status_comp(const void *a, const void *b) /* {{{ */
-{
-	const php_cli_server_http_response_status_code_pair *pa = (const php_cli_server_http_response_status_code_pair *) a;
-	const php_cli_server_http_response_status_code_pair *pb = (const php_cli_server_http_response_status_code_pair *) b;
-
-	if (pa->code < pb->code) {
-		return -1;
-	} else if (pa->code > pb->code) {
-		return 1;
-	}
-
-	return 0;
-} /* }}} */
-
 static const char *get_status_string(int code) /* {{{ */
 {
-	php_cli_server_http_response_status_code_pair needle, *result = NULL;
+	size_t e = (sizeof(status_map) / sizeof(php_cli_server_http_reponse_status_code_pair));
+	size_t s = 0;
 
-	needle.code = code;
-	needle.str = NULL;
-
-	result = bsearch(&needle, status_map, sizeof(status_map) / sizeof(needle), sizeof(needle), status_comp);
-
-	if (result) {
-		return result->str;
+	while (e != s) {
+		size_t c = MIN((e + s + 1) / 2, e - 1);
+		int d = status_map[c].code;
+		if (d > code) {
+			e = c;
+		} else if (d < code) {
+			s = c;
+		} else {
+			return status_map[c].str;
+		}
 	}
-
-	/* Returning NULL would require complicating append_http_status_line() to
-	 * not segfault in that case, so let's just return a placeholder, since RFC
-	 * 2616 requires a reason phrase. This is basically what a lot of other Web
-	 * servers do in this case anyway. */
-	return "Unknown Status Code";
+	return NULL;
 } /* }}} */
 
 static const char *get_template_string(int code) /* {{{ */
 {
-	size_t e = (sizeof(template_map) / sizeof(php_cli_server_http_response_status_code_pair));
+	size_t e = (sizeof(template_map) / sizeof(php_cli_server_http_reponse_status_code_pair));
 	size_t s = 0;
 
 	while (e != s) {
@@ -677,11 +623,13 @@ static void sapi_cli_server_register_variables(zval *track_vars_array TSRMLS_DC)
 
 static void sapi_cli_server_log_message(char *msg TSRMLS_DC) /* {{{ */
 {
+	struct timeval tv;
+	struct tm tm;
 	char buf[52];
-
-	if (php_cli_server_get_system_time(buf) != 0) {
-		memmove(buf, "unknown time, can't be fetched", sizeof("unknown time, can't be fetched"));
-	} else {
+	gettimeofday(&tv, NULL);
+	php_localtime_r(&tv.tv_sec, &tm);
+	php_asctime_r(&tm, buf);
+	{
 		size_t l = strlen(buf);
 		if (l > 0) {
 			buf[l - 1] = '\0';
@@ -759,9 +707,10 @@ static void php_cli_server_poller_remove(php_cli_server_poller *poller, int mode
 	if (fd == poller->max_fd) {
 		while (fd > 0) {
 			fd--;
-			if (PHP_SAFE_FD_ISSET(fd, &poller->rfds) || PHP_SAFE_FD_ISSET(fd, &poller->wfds)) {
+			if (((unsigned int *)&poller->rfds)[fd / (8 * sizeof(unsigned int))] || ((unsigned int *)&poller->wfds)[fd / (8 * sizeof(unsigned int))]) {
 				break;
 			}
+			fd -= fd % (8 * sizeof(unsigned int));
 		}
 		poller->max_fd = fd;
 	}
@@ -820,20 +769,23 @@ static int php_cli_server_poller_iter_on_active(php_cli_server_poller *poller, v
 	}
 
 #else
-	php_socket_t fd;
+	php_socket_t fd = 0;
 	const php_socket_t max_fd = poller->max_fd;
-
-	for (fd=0 ; fd<=max_fd ; fd++)  {
-		if (PHP_SAFE_FD_ISSET(fd, &poller->active.rfds)) {
-                if (SUCCESS != callback(opaque, fd, POLLIN)) {
-                    retval = FAILURE;
-                }
-		}
-		if (PHP_SAFE_FD_ISSET(fd, &poller->active.wfds)) {
-                if (SUCCESS != callback(opaque, fd, POLLOUT)) {
-                    retval = FAILURE;
-                }
+	const unsigned int *pr = (unsigned int *)&poller->active.rfds,
+	                   *pw = (unsigned int *)&poller->active.wfds,
+	                   *e = pr + (max_fd + (8 * sizeof(unsigned int)) - 1) / (8 * sizeof(unsigned int));
+	unsigned int mask;
+	while (pr < e && fd <= max_fd) {
+		for (mask = 1; mask; mask <<= 1, fd++) {
+			int events = (*pr & mask ? POLLIN: 0) | (*pw & mask ? POLLOUT: 0);
+			if (events) {
+				if (SUCCESS != callback(opaque, fd, events)) {
+					retval = FAILURE;
+				}
+			}
 		}
+		pr++;
+		pw++;
 	}
 #endif
 	return retval;
@@ -1325,7 +1277,7 @@ static void php_cli_server_request_translate_vpath(php_cli_server_request *reque
 	static const char *index_files[] = { "index.php", "index.html", NULL };
 	char *buf = safe_pemalloc(1, request->vpath_len, 1 + document_root_len + 1 + sizeof("index.html"), 1);
 	char *p = buf, *prev_path = NULL, *q, *vpath;
-	size_t prev_path_len = 0;
+	size_t prev_path_len;
 	int  is_static_file = 0;
 
 	if (!buf) {
@@ -2439,12 +2391,12 @@ int do_cli_server(int argc, char **argv TSRMLS_DC) /* {{{ */
 	sapi_module.phpinfo_as_text = 0;
 
 	{
+		struct timeval tv;
+		struct tm tm;
 		char buf[52];
-
-		if (php_cli_server_get_system_time(buf) != 0) {
-			memmove(buf, "unknown time, can't be fetched", sizeof("unknown time, can't be fetched"));
-		}
-
+		gettimeofday(&tv, NULL);
+		php_localtime_r(&tv.tv_sec, &tm);
+		php_asctime_r(&tm, buf);
 		printf("PHP %s Development Server started at %s"
 				"Listening on http://%s\n"
 				"Document root is %s\n"
