@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -375,16 +375,12 @@ static const http_error http_error_codes[] = {
 	{413, "Request Entity Too Large"},
 	{414, "Request-URI Too Large"},
 	{415, "Unsupported Media Type"},
-	{428, "Precondition Required"},
-	{429, "Too Many Requests"},
-	{431, "Request Header Fields Too Large"},
 	{500, "Internal Server Error"},
 	{501, "Not Implemented"},
 	{502, "Bad Gateway"},
 	{503, "Service Unavailable"},
 	{504, "Gateway Time-out"},
 	{505, "HTTP Version not supported"},
-	{511, "Network Authentication Required"},
 	{0,   NULL}
 };
 
@@ -1098,7 +1094,7 @@ static void init_request_info(TSRMLS_D)
 
 #define APACHE_PROXY_FCGI_PREFIX "proxy:fcgi://"
 		/* Fix proxy URLs in SCRIPT_FILENAME generated by Apache mod_proxy_fcgi:
-		 *     proxy:fcgi://localhost:9000/some-dir/info.php/test?foo=bar
+		 *     proxy:fcgi://localhost:9000/some-dir/info.php/test
 		 * should be changed to:
 		 *     /some-dir/info.php/test
 		 * See: http://bugs.php.net/bug.php?id=54152
@@ -1118,11 +1114,6 @@ static void init_request_info(TSRMLS_D)
 				memmove(env_script_filename, p, strlen(p) + 1);
 				apache_was_here = 1;
 			}
-			/* ignore query string if sent by Apache (RewriteRule) */
-			p = strchr(env_script_filename, '?');
-			if (p) {
-				*p =0;
-			}
 		}
 
 		if (CGIG(fix_pathinfo)) {
@@ -1142,16 +1133,13 @@ static void init_request_info(TSRMLS_D)
 				TRANSLATE_SLASHES(env_document_root);
 			}
 
-			if (!apache_was_here && env_path_translated != NULL && env_redirect_url != NULL &&
+			if (env_path_translated != NULL && env_redirect_url != NULL &&
 			    env_path_translated != script_path_translated &&
 			    strcmp(env_path_translated, script_path_translated) != 0) {
 				/*
 				 * pretty much apache specific.  If we have a redirect_url
 				 * then our script_filename and script_name point to the
 				 * php executable
-				 * we don't want to do this for the new mod_proxy_fcgi approach,
-				 * where redirect_url may also exist but the below will break
-				 * with rewrites to PATH_INFO, hence the !apache_was_here check
 				 */
 				script_path_translated = env_path_translated;
 				/* we correct SCRIPT_NAME now in case we don't have PATH_INFO */
@@ -1181,123 +1169,119 @@ static void init_request_info(TSRMLS_D)
 				int len = script_path_translated_len;
 				char *ptr;
 
-				if (pt) {
-					while ((ptr = strrchr(pt, '/')) || (ptr = strrchr(pt, '\\'))) {
-						*ptr = 0;
-						if (stat(pt, &st) == 0 && S_ISREG(st.st_mode)) {
-							/*
-							 * okay, we found the base script!
-							 * work out how many chars we had to strip off;
-							 * then we can modify PATH_INFO
-							 * accordingly
-							 *
-							 * we now have the makings of
-							 * PATH_INFO=/test
-							 * SCRIPT_FILENAME=/docroot/info.php
-							 *
-							 * we now need to figure out what docroot is.
-							 * if DOCUMENT_ROOT is set, this is easy, otherwise,
-							 * we have to play the game of hide and seek to figure
-							 * out what SCRIPT_NAME should be
-							 */
-							int ptlen = strlen(pt);
-							int slen = len - ptlen;
-							int pilen = env_path_info ? strlen(env_path_info) : 0;
-							int tflag = 0;
-							char *path_info;
-							if (apache_was_here) {
-								/* recall that PATH_INFO won't exist */
-								path_info = script_path_translated + ptlen;
-								tflag = (slen != 0 && (!orig_path_info || strcmp(orig_path_info, path_info) != 0));
-							} else {
-								path_info = env_path_info ? env_path_info + pilen - slen : NULL;
-								tflag = (orig_path_info != path_info);
-							}
-
-							if (tflag) {
-								if (orig_path_info) {
-									char old;
-
-									_sapi_cgibin_putenv("ORIG_PATH_INFO", orig_path_info TSRMLS_CC);
-									old = path_info[0];
-									path_info[0] = 0;
-									if (!orig_script_name ||
-										strcmp(orig_script_name, env_path_info) != 0) {
-										if (orig_script_name) {
-											_sapi_cgibin_putenv("ORIG_SCRIPT_NAME", orig_script_name TSRMLS_CC);
-										}
-										SG(request_info).request_uri = _sapi_cgibin_putenv("SCRIPT_NAME", env_path_info TSRMLS_CC);
-									} else {
-										SG(request_info).request_uri = orig_script_name;
-									}
-									path_info[0] = old;
-								}
-								env_path_info = _sapi_cgibin_putenv("PATH_INFO", path_info TSRMLS_CC);
-							}
-							if (!orig_script_filename ||
-								strcmp(orig_script_filename, pt) != 0) {
-								if (orig_script_filename) {
-									_sapi_cgibin_putenv("ORIG_SCRIPT_FILENAME", orig_script_filename TSRMLS_CC);
-								}
-								script_path_translated = _sapi_cgibin_putenv("SCRIPT_FILENAME", pt TSRMLS_CC);
-							}
-							TRANSLATE_SLASHES(pt);
-
-							/* figure out docroot
-							 * SCRIPT_FILENAME minus SCRIPT_NAME
-							 */
-							if (env_document_root) {
-								int l = strlen(env_document_root);
-								int path_translated_len = 0;
-								char *path_translated = NULL;
-
-								if (l && env_document_root[l - 1] == '/') {
-									--l;
-								}
-
-								/* we have docroot, so we should have:
-								 * DOCUMENT_ROOT=/docroot
-								 * SCRIPT_FILENAME=/docroot/info.php
-								 */
-
-								/* PATH_TRANSLATED = DOCUMENT_ROOT + PATH_INFO */
-								path_translated_len = l + (env_path_info ? strlen(env_path_info) : 0);
-								path_translated = (char *) emalloc(path_translated_len + 1);
-								memcpy(path_translated, env_document_root, l);
-								if (env_path_info) {
-									memcpy(path_translated + l, env_path_info, (path_translated_len - l));
-								}
-								path_translated[path_translated_len] = '\0';
-								if (orig_path_translated) {
-									_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
-								}
-								env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
-								efree(path_translated);
-							} else if (	env_script_name &&
-										strstr(pt, env_script_name)
-							) {
-								/* PATH_TRANSLATED = PATH_TRANSLATED - SCRIPT_NAME + PATH_INFO */
-								int ptlen = strlen(pt) - strlen(env_script_name);
-								int path_translated_len = ptlen + (env_path_info ? strlen(env_path_info) : 0);
-								char *path_translated = NULL;
-
-								path_translated = (char *) emalloc(path_translated_len + 1);
-								memcpy(path_translated, pt, ptlen);
-								if (env_path_info) {
-									memcpy(path_translated + ptlen, env_path_info, path_translated_len - ptlen);
-								}
-								path_translated[path_translated_len] = '\0';
-								if (orig_path_translated) {
-									_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
-								}
-								env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
-								efree(path_translated);
-							}
-							break;
+				while ((ptr = strrchr(pt, '/')) || (ptr = strrchr(pt, '\\'))) {
+					*ptr = 0;
+					if (stat(pt, &st) == 0 && S_ISREG(st.st_mode)) {
+						/*
+						 * okay, we found the base script!
+						 * work out how many chars we had to strip off;
+						 * then we can modify PATH_INFO
+						 * accordingly
+						 *
+						 * we now have the makings of
+						 * PATH_INFO=/test
+						 * SCRIPT_FILENAME=/docroot/info.php
+						 *
+						 * we now need to figure out what docroot is.
+						 * if DOCUMENT_ROOT is set, this is easy, otherwise,
+						 * we have to play the game of hide and seek to figure
+						 * out what SCRIPT_NAME should be
+						 */
+						int ptlen = strlen(pt);
+						int slen = len - ptlen;
+						int pilen = env_path_info ? strlen(env_path_info) : 0;
+						int tflag = 0;
+						char *path_info;
+						if (apache_was_here) {
+							/* recall that PATH_INFO won't exist */
+							path_info = script_path_translated + ptlen;
+							tflag = (slen != 0 && (!orig_path_info || strcmp(orig_path_info, path_info) != 0));
+						} else {
+							path_info = env_path_info ? env_path_info + pilen - slen : NULL;
+							tflag = (orig_path_info != path_info);
 						}
+
+						if (tflag) {
+							if (orig_path_info) {
+								char old;
+
+								_sapi_cgibin_putenv("ORIG_PATH_INFO", orig_path_info TSRMLS_CC);
+								old = path_info[0];
+								path_info[0] = 0;
+								if (!orig_script_name ||
+									strcmp(orig_script_name, env_path_info) != 0) {
+									if (orig_script_name) {
+										_sapi_cgibin_putenv("ORIG_SCRIPT_NAME", orig_script_name TSRMLS_CC);
+									}
+									SG(request_info).request_uri = _sapi_cgibin_putenv("SCRIPT_NAME", env_path_info TSRMLS_CC);
+								} else {
+									SG(request_info).request_uri = orig_script_name;
+								}
+								path_info[0] = old;
+							}
+							env_path_info = _sapi_cgibin_putenv("PATH_INFO", path_info TSRMLS_CC);
+						}
+						if (!orig_script_filename ||
+							strcmp(orig_script_filename, pt) != 0) {
+							if (orig_script_filename) {
+								_sapi_cgibin_putenv("ORIG_SCRIPT_FILENAME", orig_script_filename TSRMLS_CC);
+							}
+							script_path_translated = _sapi_cgibin_putenv("SCRIPT_FILENAME", pt TSRMLS_CC);
+						}
+						TRANSLATE_SLASHES(pt);
+
+						/* figure out docroot
+						 * SCRIPT_FILENAME minus SCRIPT_NAME
+						 */
+						if (env_document_root) {
+							int l = strlen(env_document_root);
+							int path_translated_len = 0;
+							char *path_translated = NULL;
+
+							if (l && env_document_root[l - 1] == '/') {
+								--l;
+							}
+
+							/* we have docroot, so we should have:
+							 * DOCUMENT_ROOT=/docroot
+							 * SCRIPT_FILENAME=/docroot/info.php
+							 */
+
+							/* PATH_TRANSLATED = DOCUMENT_ROOT + PATH_INFO */
+							path_translated_len = l + (env_path_info ? strlen(env_path_info) : 0);
+							path_translated = (char *) emalloc(path_translated_len + 1);
+							memcpy(path_translated, env_document_root, l);
+							if (env_path_info) {
+								memcpy(path_translated + l, env_path_info, (path_translated_len - l));
+							}
+							path_translated[path_translated_len] = '\0';
+							if (orig_path_translated) {
+								_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
+							}
+							env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
+							efree(path_translated);
+						} else if (	env_script_name &&
+									strstr(pt, env_script_name)
+						) {
+							/* PATH_TRANSLATED = PATH_TRANSLATED - SCRIPT_NAME + PATH_INFO */
+							int ptlen = strlen(pt) - strlen(env_script_name);
+							int path_translated_len = ptlen + (env_path_info ? strlen(env_path_info) : 0);
+							char *path_translated = NULL;
+
+							path_translated = (char *) emalloc(path_translated_len + 1);
+							memcpy(path_translated, pt, ptlen);
+							if (env_path_info) {
+								memcpy(path_translated + ptlen, env_path_info, path_translated_len - ptlen);
+							}
+							path_translated[path_translated_len] = '\0';
+							if (orig_path_translated) {
+								_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
+							}
+							env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
+							efree(path_translated);
+						}
+						break;
 					}
-				} else {
-					ptr = NULL;
 				}
 				if (!ptr) {
 					/*
@@ -1326,7 +1310,7 @@ static void init_request_info(TSRMLS_D)
 					efree(pt);
 				}
 			} else {
-				/* make sure original values are remembered in ORIG_ copies if we've changed them */
+				/* make sure path_info/translated are empty */
 				if (!orig_script_filename ||
 					(script_path_translated != orig_script_filename &&
 					strcmp(script_path_translated, orig_script_filename) != 0)) {
@@ -1335,9 +1319,7 @@ static void init_request_info(TSRMLS_D)
 					}
 					script_path_translated = _sapi_cgibin_putenv("SCRIPT_FILENAME", script_path_translated TSRMLS_CC);
 				}
-				if (!apache_was_here && env_redirect_url) {
-					/* if we used PATH_TRANSLATED to work around Apache mod_fastcgi (but not mod_proxy_fcgi,
-					 * hence !apache_was_here) weirdness, strip info accordingly */
+				if (env_redirect_url) {
 					if (orig_path_info) {
 						_sapi_cgibin_putenv("ORIG_PATH_INFO", orig_path_info TSRMLS_CC);
 						_sapi_cgibin_putenv("PATH_INFO", NULL TSRMLS_CC);
@@ -1726,9 +1708,9 @@ int main(int argc, char *argv[])
 				SG(request_info).no_headers = 1;
 
 #if ZEND_DEBUG
-				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2014 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__,        __TIME__, get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2012 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__,        __TIME__, get_zend_version());
 #else
-				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2014 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__,      get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2012 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__,      get_zend_version());
 #endif
 				php_request_shutdown((void *) 0);
 				fcgi_shutdown();
